// Compatibility layer with allegro

#include <SDL.h>
#include "raine.h"
#include <sys/stat.h>

void sdl_init() {
  /* Initialize the SDL library */
  if ( SDL_Init(/* SDL_INIT_AUDIO| */SDL_INIT_VIDEO) < 0 ) {
    fprintf(stderr, "Couldn't initialize SDL: %s\n",SDL_GetError());
    exit(2);
  }
  atexit(SDL_Quit);
}

void sdl_done() {
  // This should also save the sdl specific params... Later !
  SDL_Quit();
}

BITMAP *surface_to_bmp(SDL_Surface *s) {
  int a;
  BITMAP *bmp;

  bmp = malloc( sizeof(BITMAP) + sizeof(char*) * s->h);
  bmp->vtable = (void*)s;
  bmp->w = s->w; bmp->h = s->h;
  bmp->x_ofs = bmp->y_ofs = 0;
  for (a=0; a < s->h; a++)
    bmp->line[a] = (UINT8 *)s->pixels+a*s->pitch;
  return bmp;
}

BITMAP *sdl_create_bitmap_ex(int bpp, int w, int h) {
  // Init a fake bitmap to point to a newly created sdl_surface
  int r,g,b,a; // masks if necessary...
  SDL_Surface *s;

  if (bpp == 8) {
    r = g = b = a = 0; // Useless
  } else {
    const SDL_VideoInfo *inf = SDL_GetVideoInfo();
    SDL_PixelFormat *fmt = inf->vfmt;
    if (fmt->BitsPerPixel != bpp) {
      fprintf(stderr,"Something went wrong in bitmap creation\n");
      SDL_Quit();
    }
    r = fmt->Rmask;
    g = fmt->Gmask;
    b = fmt->Bmask;
    a = fmt->Amask;
  }

  s = SDL_CreateRGBSurface(SDL_SWSURFACE, // these bitmaps are ALWAYS in ram
			   w,h,bpp,r,g,b,a);
/*   if ( SDL_MUSTLOCK(screen) ) { */
/*     fprintf(stderr,"Locing problem !\n"); */
/*     SDL_Quit(-1); */
/*   } */

  return surface_to_bmp(s);
}

void sdl_destroy_bitmap(BITMAP *bmp) {
  if (bmp) {
    if (bmp->vtable) { // real bitmap (not sub bitmap)
      free(bmp->line);
      SDL_FreeSurface((SDL_Surface*)bmp->vtable);
    }
    free(bmp);
  }
}

// Strings...

void put_backslash(char *s) {
  int l = strlen(s)-1;
  s += l;
  if (*s != '/' && *s != '\\')
    strcat(s,SLASH);
}

// Todo...

int  save_pcx(const char *filename, BITMAP *bmp, const RGB
	      *pal) {
  // This could be the save_pcx taken from allegro,
  // another saving function eventually to another format
  // or nothing. For now : nothing.
  return 0;
}

void clear_game_screen() {
  SDL_Rect r;
  r.x = r.y = 0;
  r.w = GameViewBitmap->w;
  r.h = GameViewBitmap->h;
  SDL_FillRect(GameViewBitmap,&r,0);
}
