/* Raine line scroll. Made for cave driver : 16x16 sprites, 16bpp,
   and offsets of the lines every 4 bytes */

#include "deftypes.h"
#include "raine.h"

extern int disp_x_16;

void ldraw16x16_Mapped(UINT8 *SPR, int x, int y, UINT8 *cmap, UINT8 *dy)
{
   UINT8 *line;
   INT16 xx,yy,dx;
   for(yy=0; yy<16; yy++){
     dx = x+ReadWord(dy+yy*4);
     if (dx < 0 || dx > disp_x_16) {
       SPR += 16;
       continue;
     }
     line = ((UINT8 *)(GameBitmap->line[y+yy]))+ dx;
      for(xx=0; xx<16; xx++){
         *line++ = ((UINT8 *)cmap)[ *SPR++ ];
      }
   }
}

// The column draw is designed to replace the line scroll when the screen
// is rotated by 270. If a game uses 90 or 180 we will use 2 other famillies
// of functions.
// This cdraw16x16 is very unefficient compared to the usual Draw16x16
// functions because :
// - the line scroll indexes follow y (on screen)
// - the sprite follows x.
// This whole thing should probably be entierly rewritten, and preferably using
// a macro this time to have only 1 version for 8,16 and 32 bpp.

void cdraw16x16_Mapped(UINT8 *SPR, int x, int y, UINT8 *cmap, UINT8 *dx)
{
  /* Column draw, or line scroll rotated by 270 */
   UINT8 *line;
   INT16 myx,xx,yy,dy;
   for(xx=0; xx<=15; xx++){
     dy = y-ReadWord(dx+xx*4);
     if (dy < 0 || dy > disp_x_16) {
       // fprintf(stderr,"%d > %d (%d)\n",dy,disp_x_16,disp_x_16);
       continue;
     }
     myx = x + xx;
     for(yy=0; yy<16; yy++){
       UINT8 *spr = &SPR[xx+16*yy];
       line = ((UINT8 *)(GameBitmap->line[dy+yy]))+myx ;
       *line = ((UINT8 *)cmap)[ *spr ];
     }
   }
}

void ldraw16x16_Mapped_FlipX(UINT8 *SPR, INT16 x, INT16 y, UINT8 *cmap, UINT8 *dy)
{
   UINT8 *line;
   INT16 xx,yy,dx;

   for(yy=15; yy>=0; yy--){
     dx = x+ReadWord(dy+yy*4);
     if (dx < 0 || dx > disp_x_16) {
       SPR += 16;
       continue;
     }
     line = ((UINT8 *)(GameBitmap->line[y+yy]))+ dx;
     for(xx=0; xx<16; xx++){
       *line++ = ((UINT8 *)cmap)[ *SPR++ ];
     }
   }
}

void cdraw16x16_Mapped_FlipX(UINT8 *SPR, int x, int y, UINT8 *cmap, UINT8 *dx)
{
  /* Column draw, or line scroll rotated by 270 */
   UINT8 *line;
   INT16 myx,xx,yy,dy;
   for(xx=0; xx<=15; xx++){
     dy = y-ReadWord(dx+xx*4);
     if (dy < 0 || dy > disp_x_16) {
       continue;
     }
     myx = x + xx;
     for(yy=15; yy>=0; yy--){
       UINT8 *spr = &SPR[xx+16*(16-yy)];
       line = ((UINT8 *)(GameBitmap->line[dy+yy]))+myx ;
       *line = ((UINT8 *)cmap)[ *spr ];
     }
   }
}

void ldraw16x16_Mapped_FlipY(UINT8 *SPR, int x, int y, UINT8 *cmap, UINT8 *dy)
{
   UINT8 *line;
   INT16 xx,yy,dx;

   for(yy=0; yy<16; yy++){
     dx = x+ReadWord(dy+yy*4);
     if (dx < 0 || dx > disp_x_16) {
       SPR += 16;
       continue;
     }
     line = ((UINT8 *)(GameBitmap->line[y+yy])) + dx+15;
     for(xx=0; xx<16; xx++){
       *line-- = ((UINT8 *)cmap)[ *SPR++ ];
     }
   }
}

void cdraw16x16_Mapped_FlipY(UINT8 *SPR, int x, int y, UINT8 *cmap, UINT8 *dx)
{
  /* Column draw, or line scroll rotated by 270 */
   UINT8 *line;
   INT16 myx,xx,yy,dy;
   for(xx=15; xx>=0; xx--){
     dy = y-ReadWord(dx+xx*4);
     if (dy < 0 || dy > disp_x_16) {
       continue;
     }
     myx = x + xx;
     for(yy=0; yy<16; yy++){
       UINT8 *spr = &SPR[16-xx+16*yy];
       line = ((UINT8 *)(GameBitmap->line[dy+yy]))+myx ;
       *line = ((UINT8 *)cmap)[ *spr ];
     }
   }
}

void ldraw16x16_Mapped_FlipXY(UINT8 *SPR, int x, int y, UINT8 *cmap, UINT8 *dy)
{
   UINT8 *line;
   INT16 xx,yy,dx;

   for(yy=15; yy>=0; yy--){
     dx = x+ReadWord(dy+yy*4);
     if (dx < 0 || dx > disp_x_16) {
       SPR += 16;
       continue;
     }
     line = ((UINT8 *)(GameBitmap->line[y+yy])) + dx + 15;
     for(xx=0; xx<16; xx++){
       *line-- = ((UINT8 *)cmap)[ *SPR++ ];
     }
   }
}

void cdraw16x16_Mapped_FlipXY(UINT8 *SPR, int x, int y, UINT8 *cmap, UINT8 *dx)
{
  /* Column draw, or line scroll rotated by 270 */
   UINT8 *line;
   INT16 myx,xx,yy,dy;
   for(xx=15; xx>=0; xx--){
     dy = y-ReadWord(dx+xx*4);
     if (dy < 0 || dy > disp_x_16) {
       continue;
     }
     myx = x + xx;
     for(yy=15; yy>=0; yy--){
       UINT8 *spr = &SPR[16-xx+16*(16-yy)];
       line = ((UINT8 *)(GameBitmap->line[dy+yy]))+myx ;
       *line = ((UINT8 *)cmap)[ *spr ];
     }
   }
}

// 16x16 mapped transparent sprites

void ldraw16x16_Trans_Mapped(UINT8 *SPR, int x, int y, UINT8 *cmap,UINT8 *dy)
{
   UINT8 *line;
   INT16 xx,yy,dx;

   for(yy=0; yy<16; yy++){
     dx = x+ReadWord(dy+yy*4);
     if (dx < 0 || dx > disp_x_16) {
       SPR += 16;
       continue;
     }
     line = ((UINT8 *)(GameBitmap->line[y+yy])) + dx;
     for(xx=0; xx<16; xx++, SPR++){
       if(*SPR)
	 line[xx] = ((UINT8 *)cmap)[ *SPR ];
     }
   }
}

void cdraw16x16_Trans_Mapped(UINT8 *SPR, int x, int y, UINT8 *cmap, UINT8 *dx)
{
  /* Column draw, or line scroll rotated by 270 */
   UINT8 *line;
   INT16 myx,xx,yy,dy;
   for(xx=0; xx<=15; xx++){
     dy = y-ReadWord(dx+xx*4);
     if (dy < 0 || dy > disp_x_16) {
       continue;
     }
     myx = x + xx;
     for(yy=0; yy<16; yy++){
       UINT8 *spr = &SPR[xx+16*yy];
       if(*spr) {
	 line = ((UINT8 *)(GameBitmap->line[dy+yy]))+myx ;
	 *line = ((UINT8 *)cmap)[ *spr ];
       }
     }
   }
}

void ldraw16x16_Trans_Mapped_FlipX(UINT8 *SPR, int x, int y, UINT8 *cmap, UINT8 *dy)
{
   UINT8 *line;
   INT16 xx,yy,dx;

   for(yy=15; yy>=0; yy--){
     dx = x+ReadWord(dy+yy*4);
     if (dx < 0 || dx > disp_x_16) {
       SPR += 16;
       continue;
     }
     line = ((UINT8 *)(GameBitmap->line[y+yy])) + dx;
     for(xx=0; xx<16; xx++, SPR++){
       if(*SPR)
	 line[xx] = ((UINT8 *)cmap)[ *SPR ];
     }
   }
}

void cdraw16x16_Trans_Mapped_FlipX(UINT8 *SPR, int x, int y, UINT8 *cmap, UINT8 *dx)
{
  /* Column draw, or line scroll rotated by 270 */
   UINT8 *line;
   INT16 myx,xx,yy,dy;
   for(xx=0; xx<=15; xx++){
     dy = y-ReadWord(dx+xx*4);
     if (dy < 0 || dy > disp_x_16) {
       continue;
     }
     myx = x + xx;
     for(yy=15; yy>=0; yy--){
       UINT8 *spr = &SPR[xx+16*(16-yy)];
       if (*spr) {
	 line = ((UINT8 *)(GameBitmap->line[dy+yy]))+myx ;
	 *line = ((UINT8 *)cmap)[ *spr ];
       }
     }
   }
}

void ldraw16x16_Trans_Mapped_FlipY(UINT8 *SPR, int x, int y, UINT8 *cmap, UINT8 *dy)
{
   UINT8 *line;
   INT16 xx,yy,dx;

   for(yy=0; yy<16; yy++){
     dx = x+ReadWord(dy+yy*4);
     if (dx < 0 || dx > disp_x_16) {
       SPR += 16;
       continue;
     }
     line = ((UINT8 *)(GameBitmap->line[y+yy])) + dx;
     for(xx=15; xx>=0; xx--, SPR++){
       if(*SPR)
	 line[xx] = ((UINT8 *)cmap)[ *SPR ];
     }
   }
}

void cdraw16x16_Trans_Mapped_FlipY(UINT8 *SPR, int x, int y, UINT8 *cmap, UINT8 *dx)
{
  /* Column draw, or line scroll rotated by 270 */
   UINT8 *line;
   INT16 myx,xx,yy,dy;
   for(xx=15; xx>=0; xx--){
     dy = y-ReadWord(dx+xx*4);
     if (dy < 0 || dy > disp_x_16) {
       continue;
     }
     myx = x + xx;
     for(yy=0; yy<16; yy++){
       UINT8 *spr = &SPR[16-xx+16*(yy)];
       if (*spr) {
	 line = ((UINT8 *)(GameBitmap->line[dy+yy]))+myx ;
	 *line = ((UINT8 *)cmap)[ *spr ];
       }
     }
   }
}

void ldraw16x16_Trans_Mapped_FlipXY(UINT8 *SPR, int x, int y, UINT8 *cmap, UINT8 *dy)
{
   UINT8 *line;
   INT16 xx,yy,dx;

   for(yy=15; yy>=0; yy--){
     dx = x+ReadWord(dy+yy*4);
     if (dx < 0 || dx > disp_x_16) {
       SPR += 16;
       continue;
     }
     line = ((UINT8 *)(GameBitmap->line[y+yy])) + dx;
     for(xx=15; xx>=0; xx--, SPR++){
       if(*SPR)
	 line[xx] = ((UINT8 *)cmap)[ *SPR ];
     }
   }
}

void cdraw16x16_Trans_Mapped_FlipXY(UINT8 *SPR, int x, int y, UINT8 *cmap, UINT8 *dx)
{
  /* Column draw, or line scroll rotated by 270 */
   UINT8 *line;
   INT16 myx,xx,yy,dy;
   for(xx=15; xx>=0; xx--){
     dy = y-ReadWord(dx+xx*4);
     if (dy < 0 || dy > disp_x_16) {
       continue;
     }
     myx = x + xx;
     for(yy=15; yy>=0; yy--){
       UINT8 *spr = &SPR[16-xx+16*(16-yy)];
       if (*spr) {
	 line = ((UINT8 *)(GameBitmap->line[dy+yy]))+myx ;
	 *line = ((UINT8 *)cmap)[ *spr ];
       }
     }
   }
}

// 16x16 sprites (I wonder if these will ever be used).
// Since I don't feel courageous, I am not going to write the bound
// checking code for real sprites...

// This is really a piece of history. The only game I know which uses
// direct mapping is bubble bobble !

void ldraw16x16(UINT8 *SPR, int x, int y, UINT8 cmap, UINT8 *dy)
{
   UINT8 *line;
   INT16 xx,yy;

   for(yy=0; yy<16; yy++){
      line = ((UINT8 *)(GameBitmap->line[y+yy])) + x + ReadWord(dy+yy*4);
      for(xx=0; xx<16; xx++){
         *line++ = *SPR++ | cmap;
      }
   }
}

void ldraw16x16_FlipX(UINT8 *SPR, int x, int y, UINT8 cmap, UINT8 *dy)
{
   UINT8 *line;
   INT16 xx,yy;

   for(yy=15; yy>=0; yy--){
      line = ((UINT8 *)(GameBitmap->line[y+yy])) + x + ReadWord(dy+yy*4);
      for(xx=0; xx<16; xx++){
         *line++ = *SPR++ | cmap;
      }
   }
}

void ldraw16x16_FlipY(UINT8 *SPR, int x, int y, UINT8 cmap, UINT8 *dy)
{
   UINT8 *line;
   INT16 xx,yy;

   for(yy=0; yy<16; yy++){
      line = ((UINT8 *)(GameBitmap->line[y+yy])) + x + 7 + ReadWord(dy+yy*4);
      for(xx=0; xx<16; xx++){
         *line-- = *SPR++ | cmap;
      }
   }
}

void ldraw16x16_FlipXY(UINT8 *SPR, int x, int y, UINT8 cmap, UINT8 *dy)
{
   UINT8 *line;
   INT16 xx,yy;

   for(yy=15; yy>=0; yy--){
      line = ((UINT8 *)(GameBitmap->line[y+yy])) + x + 7 + ReadWord(dy+yy*4);
      for(xx=0; xx<16; xx++){
         *line-- = *SPR++ | cmap;
      }
   }
}

// 16x16 transparent sprites

void ldraw16x16_Trans(UINT8 *SPR, int x, int y, UINT8 cmap, UINT8 *dy)
{
   UINT8 *line;
   INT16 xx,yy;

   for(yy=0; yy<16; yy++){
      line = ((UINT8 *)(GameBitmap->line[y+yy])) + x + ReadWord(dy+yy*4);
      for(xx=0; xx<16; xx++, SPR++){
         if(*SPR)
            line[xx] = *SPR | cmap;
      }
   }
}

void ldraw16x16_Trans_FlipX(UINT8 *SPR, int x, int y, UINT8 cmap, UINT8 *dy)
{
   UINT8 *line;
   INT16 xx,yy;

   for(yy=15; yy>=0; yy--){
      line = ((UINT8 *)(GameBitmap->line[y+yy])) + x + ReadWord(dy+yy*4);
      for(xx=0; xx<16; xx++, SPR++){
         if(*SPR)
            line[xx] = *SPR | cmap;
      }
   }
}

void ldraw16x16_Trans_FlipY(UINT8 *SPR, int x, int y, UINT8 cmap, UINT8 *dy)
{
   UINT8 *line;
   INT16 xx,yy;

   for(yy=0; yy<16; yy++){
      line = ((UINT8 *)(GameBitmap->line[y+yy])) + x + ReadWord(dy+yy*4);
      for(xx=15; xx>=0; xx--, SPR++){
         if(*SPR)
            line[xx] = *SPR | cmap;
      }
   }
}

void ldraw16x16_Trans_FlipXY(UINT8 *SPR, int x, int y, UINT8 cmap, UINT8 *dy)
{
   UINT8 *line;
   INT16 xx,yy;

   for(yy=15; yy>=0; yy--){
      line = ((UINT8 *)(GameBitmap->line[y+yy])) + x + ReadWord(dy+yy*4);
      for(xx=15; xx>=0; xx--, SPR++){
         if(*SPR)
            line[xx] = *SPR | cmap;
      }
   }
}
